import numpy as np

from flatland.core.grid.grid_utils import Vec2dOperations as Vec2d
from flatland.envs.observations import GlobalObsForRailEnv
from flatland.envs.rail_env import RailEnv
from flatland.envs.rail_generators_city_generator import city_generator
from flatland.envs.schedule_generators import city_schedule_generator


def test_city_generator():
    dist_fun = Vec2d.get_manhattan_distance
    env = RailEnv(width=50,
                  height=50,
                  rail_generator=city_generator(num_cities=5,
                                                city_size=10,
                                                allowed_rotation_angles=[90],
                                                max_number_of_station_tracks=4,
                                                nbr_of_switches_per_station_track=2,
                                                connect_max_nbr_of_shortes_city=2,
                                                do_random_connect_stations=False,
                                                a_star_distance_function=dist_fun,
                                                seed=0,
                                                print_out_info=False
                                                ),
                  schedule_generator=city_schedule_generator(),
                  number_of_agents=10,
                  obs_builder_object=GlobalObsForRailEnv())

    expected_grid_map = np.zeros((50, 50), dtype=env.rail.transitions.get_type())

    expected_grid_map[8][16]=4
    expected_grid_map[8][17]=5633
    expected_grid_map[8][18]=1025
    expected_grid_map[8][19]=1025
    expected_grid_map[8][20]=17411
    expected_grid_map[8][21]=1025
    expected_grid_map[8][22]=1025
    expected_grid_map[8][23]=1025
    expected_grid_map[8][24]=1025
    expected_grid_map[8][25]=1025
    expected_grid_map[8][26]=4608
    expected_grid_map[9][16]=16386
    expected_grid_map[9][17]=50211
    expected_grid_map[9][18]=1025
    expected_grid_map[9][19]=1025
    expected_grid_map[9][20]=3089
    expected_grid_map[9][21]=1025
    expected_grid_map[9][22]=256
    expected_grid_map[9][26]=32800
    expected_grid_map[10][6]=16386
    expected_grid_map[10][7]=1025
    expected_grid_map[10][8]=1025
    expected_grid_map[10][9]=1025
    expected_grid_map[10][10]=1025
    expected_grid_map[10][11]=1025
    expected_grid_map[10][12]=1025
    expected_grid_map[10][13]=1025
    expected_grid_map[10][14]=1025
    expected_grid_map[10][15]=1025
    expected_grid_map[10][16]=33825
    expected_grid_map[10][17]=34864
    expected_grid_map[10][26]=32800
    expected_grid_map[11][6]=32800
    expected_grid_map[11][16]=32800
    expected_grid_map[11][17]=32800
    expected_grid_map[11][26]=32800
    expected_grid_map[12][6]=32800
    expected_grid_map[12][16]=32800
    expected_grid_map[12][17]=32800
    expected_grid_map[12][26]=32800
    expected_grid_map[13][6]=32800
    expected_grid_map[13][16]=32800
    expected_grid_map[13][17]=32800
    expected_grid_map[13][26]=32800
    expected_grid_map[14][6]=32800
    expected_grid_map[14][16]=32800
    expected_grid_map[14][17]=32800
    expected_grid_map[14][26]=32800
    expected_grid_map[15][6]=32800
    expected_grid_map[15][16]=32800
    expected_grid_map[15][17]=32800
    expected_grid_map[15][26]=32800
    expected_grid_map[16][6]=32800
    expected_grid_map[16][16]=32800
    expected_grid_map[16][17]=32800
    expected_grid_map[16][26]=32800
    expected_grid_map[17][6]=32800
    expected_grid_map[17][16]=72
    expected_grid_map[17][17]=1097
    expected_grid_map[17][18]=1025
    expected_grid_map[17][19]=1025
    expected_grid_map[17][20]=1025
    expected_grid_map[17][21]=1025
    expected_grid_map[17][22]=1025
    expected_grid_map[17][23]=1025
    expected_grid_map[17][24]=1025
    expected_grid_map[17][25]=1025
    expected_grid_map[17][26]=33825
    expected_grid_map[17][27]=4608
    expected_grid_map[18][6]=32800
    expected_grid_map[18][26]=72
    expected_grid_map[18][27]=52275
    expected_grid_map[18][28]=5633
    expected_grid_map[18][29]=17411
    expected_grid_map[18][30]=1025
    expected_grid_map[18][31]=1025
    expected_grid_map[18][32]=256
    expected_grid_map[19][6]=32800
    expected_grid_map[19][25]=16386
    expected_grid_map[19][26]=1025
    expected_grid_map[19][27]=2136
    expected_grid_map[19][28]=1097
    expected_grid_map[19][29]=1097
    expected_grid_map[19][30]=5633
    expected_grid_map[19][31]=1025
    expected_grid_map[19][32]=256
    expected_grid_map[20][6]=32800
    expected_grid_map[20][25]=32800
    expected_grid_map[20][26]=16386
    expected_grid_map[20][27]=17411
    expected_grid_map[20][28]=1025
    expected_grid_map[20][29]=1025
    expected_grid_map[20][30]=3089
    expected_grid_map[20][31]=1025
    expected_grid_map[20][32]=256
    expected_grid_map[21][6]=32800
    expected_grid_map[21][16]=16386
    expected_grid_map[21][17]=1025
    expected_grid_map[21][18]=1025
    expected_grid_map[21][19]=1025
    expected_grid_map[21][20]=1025
    expected_grid_map[21][21]=1025
    expected_grid_map[21][22]=1025
    expected_grid_map[21][23]=1025
    expected_grid_map[21][24]=1025
    expected_grid_map[21][25]=33825
    expected_grid_map[21][26]=33825
    expected_grid_map[21][27]=2064
    expected_grid_map[22][6]=32800
    expected_grid_map[22][16]=32800
    expected_grid_map[22][25]=32800
    expected_grid_map[22][26]=32800
    expected_grid_map[23][6]=32800
    expected_grid_map[23][16]=32800
    expected_grid_map[23][25]=32800
    expected_grid_map[23][26]=32800
    expected_grid_map[24][6]=32800
    expected_grid_map[24][16]=32800
    expected_grid_map[24][25]=32800
    expected_grid_map[24][26]=32800
    expected_grid_map[25][6]=32800
    expected_grid_map[25][16]=32800
    expected_grid_map[25][25]=32800
    expected_grid_map[25][26]=32800
    expected_grid_map[26][6]=32800
    expected_grid_map[26][16]=32800
    expected_grid_map[26][25]=32800
    expected_grid_map[26][26]=32800
    expected_grid_map[27][6]=72
    expected_grid_map[27][7]=1025
    expected_grid_map[27][8]=1025
    expected_grid_map[27][9]=17411
    expected_grid_map[27][10]=1025
    expected_grid_map[27][11]=1025
    expected_grid_map[27][12]=1025
    expected_grid_map[27][13]=1025
    expected_grid_map[27][14]=1025
    expected_grid_map[27][15]=4608
    expected_grid_map[27][16]=72
    expected_grid_map[27][17]=17411
    expected_grid_map[27][18]=5633
    expected_grid_map[27][19]=1025
    expected_grid_map[27][20]=1025
    expected_grid_map[27][21]=1025
    expected_grid_map[27][22]=1025
    expected_grid_map[27][23]=1025
    expected_grid_map[27][24]=1025
    expected_grid_map[27][25]=33825
    expected_grid_map[27][26]=2064
    expected_grid_map[28][6]=4
    expected_grid_map[28][7]=1025
    expected_grid_map[28][8]=1025
    expected_grid_map[28][9]=3089
    expected_grid_map[28][10]=1025
    expected_grid_map[28][11]=1025
    expected_grid_map[28][12]=1025
    expected_grid_map[28][13]=1025
    expected_grid_map[28][14]=4608
    expected_grid_map[28][15]=72
    expected_grid_map[28][16]=1025
    expected_grid_map[28][17]=2136
    expected_grid_map[28][18]=1097
    expected_grid_map[28][19]=5633
    expected_grid_map[28][20]=5633
    expected_grid_map[28][21]=1025
    expected_grid_map[28][22]=256
    expected_grid_map[28][25]=32800
    expected_grid_map[29][6]=4
    expected_grid_map[29][7]=5633
    expected_grid_map[29][8]=20994
    expected_grid_map[29][9]=5633
    expected_grid_map[29][10]=1025
    expected_grid_map[29][11]=1025
    expected_grid_map[29][12]=1025
    expected_grid_map[29][13]=1025
    expected_grid_map[29][14]=1097
    expected_grid_map[29][15]=5633
    expected_grid_map[29][16]=1025
    expected_grid_map[29][17]=17411
    expected_grid_map[29][18]=5633
    expected_grid_map[29][19]=1097
    expected_grid_map[29][20]=3089
    expected_grid_map[29][21]=20994
    expected_grid_map[29][22]=1025
    expected_grid_map[29][23]=1025
    expected_grid_map[29][24]=1025
    expected_grid_map[29][25]=2064
    expected_grid_map[30][6]=16386
    expected_grid_map[30][7]=38505
    expected_grid_map[30][8]=3089
    expected_grid_map[30][9]=1097
    expected_grid_map[30][10]=1025
    expected_grid_map[30][11]=1025
    expected_grid_map[30][12]=256
    expected_grid_map[30][15]=32800
    expected_grid_map[30][16]=16386
    expected_grid_map[30][17]=52275
    expected_grid_map[30][18]=1097
    expected_grid_map[30][19]=1025
    expected_grid_map[30][20]=1025
    expected_grid_map[30][21]=3089
    expected_grid_map[30][22]=256
    expected_grid_map[31][6]=32800
    expected_grid_map[31][7]=32800
    expected_grid_map[31][15]=72
    expected_grid_map[31][16]=37408
    expected_grid_map[31][17]=32800
    expected_grid_map[32][6]=32800
    expected_grid_map[32][7]=32800
    expected_grid_map[32][16]=32800
    expected_grid_map[32][17]=32800
    expected_grid_map[33][6]=32800
    expected_grid_map[33][7]=32800
    expected_grid_map[33][16]=32800
    expected_grid_map[33][17]=32800
    expected_grid_map[34][6]=32800
    expected_grid_map[34][7]=32800
    expected_grid_map[34][16]=32800
    expected_grid_map[34][17]=32800
    expected_grid_map[35][6]=32800
    expected_grid_map[35][7]=32800
    expected_grid_map[35][16]=32800
    expected_grid_map[35][17]=32800
    expected_grid_map[36][6]=32800
    expected_grid_map[36][7]=32800
    expected_grid_map[36][16]=32800
    expected_grid_map[36][17]=32800
    expected_grid_map[37][6]=72
    expected_grid_map[37][7]=1097
    expected_grid_map[37][8]=1025
    expected_grid_map[37][9]=1025
    expected_grid_map[37][10]=1025
    expected_grid_map[37][11]=1025
    expected_grid_map[37][12]=1025
    expected_grid_map[37][13]=1025
    expected_grid_map[37][14]=1025
    expected_grid_map[37][15]=1025
    expected_grid_map[37][16]=33897
    expected_grid_map[37][17]=37408
    expected_grid_map[38][16]=72
    expected_grid_map[38][17]=52275
    expected_grid_map[38][18]=5633
    expected_grid_map[38][19]=17411
    expected_grid_map[38][20]=1025
    expected_grid_map[38][21]=1025
    expected_grid_map[38][22]=256
    expected_grid_map[39][16]=4
    expected_grid_map[39][17]=52275
    expected_grid_map[39][18]=3089
    expected_grid_map[39][19]=1097
    expected_grid_map[39][20]=5633
    expected_grid_map[39][21]=1025
    expected_grid_map[39][22]=256
    expected_grid_map[40][16]=4
    expected_grid_map[40][17]=1097
    expected_grid_map[40][18]=1025
    expected_grid_map[40][19]=1025
    expected_grid_map[40][20]=3089
    expected_grid_map[40][21]=1025
    expected_grid_map[40][22]=256

    assert np.array_equal(env.rail.grid, expected_grid_map), "actual={}, expected={}".format(env.rail.grid,
                                                                                             expected_grid_map)
    
    s0 = 0
    s1 = 0
    for a in range(env.get_num_agents()):
        s0 = Vec2d.get_manhattan_distance(env.agents[a].position, (0, 0))
        s1 = Vec2d.get_chebyshev_distance(env.agents[a].position, (0, 0))
    assert s0 == 58, "actual={}".format(s0)
    assert s1 == 38, "actual={}".format(s1)
